# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['controlled_vocabulary',
 'controlled_vocabulary.management',
 'controlled_vocabulary.management.commands',
 'controlled_vocabulary.migrations',
 'controlled_vocabulary.vocabularies']

package_data = \
{'': ['*'],
 'controlled_vocabulary': ['static/admin/css/*',
                           'static/admin/js/*',
                           'templates/controlled_vocabulary/*']}

install_requires = \
['django>=2.2,<3.2', 'urllib3>=1.25,<2.0']

extras_require = \
{'toml_tox': ['tox>=3.0,<4.0', 'coverage>=4.5,<5.0']}

setup_kwargs = {
    'name': 'django-controlled-vocabulary',
    'version': '0.10',
    'description': 'Link your data to authority lists or your own controlled lists',
    'long_description': '[![PyPI version](https://badge.fury.io/py/django-controlled-vocabulary.svg)](https://badge.fury.io/py/django-controlled-vocabulary) [![Build Status](https://travis-ci.org/kingsdigitallab/django-controlled-vocabulary.svg?branch=master)](https://travis-ci.org/kingsdigitallab/django-controlled-vocabulary)\n[Advanced topics (Wiki)](https://github.com/kingsdigitallab/django-controlled-vocabulary/wiki)\n\n# Django Controlled Vocabulary\n\nThis app provides models and admin interface to link your data to standard vocabularies (e.g. ISO language codes, Wikidata). Benefits: increases the consistency and understandability of your project data.\n\nRequirements: Python 3.5+, Django 2.2+\n\nDevelopment Status: **Beta**\n\n<img src="docs/img/controlled-term-widget.png" />\n\n_A ControlledTerm field in the Django admin interface. The user selects the vocabulary (here: Wikidata), then starts typing a term in the text box. Suggestions are brought from Wikidata. When the user saves the changes, information about the selected term is copied into the database (url, identifier, label)._\n\n# Features\n\n* create your own controlled lists of terms (i.e. a **local** lists, project-specific)\n* look up terms from **standard vocabularies** (i.e. **authority files** maintained by other organisations)\n* extensible **plug-in architecture** for lookups into particular vocabularies (see table below for built-in plugins)\n* **stores** used terms from remote vocabularies into your database:\n  * space efficient (doesn\'t clutter the database with unused terms)\n  * self-contained (i.e. can still works offline & DB always \'semantically\' complete)\n* **autocomplete** widget for Django admin; reusable ControlledTermField for your models\n* **command line tool** to download vocabulary files from authoritative sources\n* [TODO] possibility to store additional **metadata** (e.g. geographic coordinates)\n* [TODO] simple **rest API** to publish your own terms\n\n## Standard vocabularies included\n\nBuilt-in plugins for the following authority files:\n\n| Vocabulary | Description |\n| ------------- | ------------- |\n| [Schema.org](https://schema.org/) | High-level categories of content |\n| [Wikidata](https://www.wikidata.org) | High level concepts or specific instances (e.g. places, people) |\n| [ISO 639-2](http://id.loc.gov/vocabulary/iso639-2.html) | Language codes |\n| [DCMI Type](https://www.dublincore.org/specifications/dublin-core/dcmi-type-vocabulary/#section-7-dcmi-type-vocabulary) | Dublin Core Format Type |\n| [MIME](https://www.iana.org/assignments/media-types/media-types.xhtml) | Media/File types |\n| [FAST Topics](https://www.oclc.org/research/themes/data-science/fast.html) | Topic categorisation |\n| [FAST Forms and Genres](https://www.oclc.org/research/themes/data-science/fast.html) | Genres of a piece of work |\n| [VIAF](https://www.oclc.org/en/viaf.html) | Various: regions, people, companies, ... |\n\n# Data Model & Software Design\n\n## Django models\n\n| Vocabularies | Terms |\n| ------------- | ------------- |\n| <img src="docs/img/controlled-vocabulary-list.png" width="400" />  | <img src="docs/img/controlled-term-list.png" width="400" />  |\n\n* **ControlledVocabulary**\n  * prefix: the vocabulary standard prefix, see http://prefix.cc/wikidata\n  * label: the short name of the vocabulary\n  * base_url: the url used as a base for all terms in the vocabulary\n  * concept: the type of terms this vocabulary represents (e.g. language, people)\n  * description: a longer description\n\n* **ControlledTerm**\n  * termid: a unique code for the term within a vocabulary, it is case sensitive\n  * label: standard name for the term, as provided by the authority\n  * vocabulary: a reference to the ControlledVocabulary this term belongs to\n  \nConventions: \n  * joining base_url (e.g. http://schema.org) with termid (e.g. Movie) **must** give the exact standard/canonical URI for the term, e.g.  http://schema.org/Movie\n\n## Vocabulary plug-ins / managers\n\nA Vocabulary **plug-in** / **manager** is a python class that provides services for a vocabulary:\n* autocomplete terms from local or remote datasets (see ControlledTermField)\n* supplies metadata for the vocabulary (see ControlledVocabulary)\n\nManagers can provide terms from a CSV file downloaded from an authoritative source.\n\nSome vocabularies can contain thousands of terms or more. A plugin will\nonly insert the terms used by your application. The rest will be accessed on\ndemand from a file on disk or in a third-party server. This approach saves\ndatabase space and keeps your application data self-contained.\n\nThis project comes with built-in plugins such a Wikidata or Schema.org. Those plugins are **enabled** by default; see below how to selectively enable them.\n\nThis architecture allows third-party plugins to be supplied via separate\npython packages.\n\n# Limitations\n* **controlled list** rather than fully fledged vocabularies, (i.e. just a bag of terms with unique IDs/URIs, no support for taxonomic relationships among terms like broader, narrower, synonyms, ...)\n* no notion of granularity (e.g. geonames country, region, city, street are all treated as part of the same vocabulary)\n\n# Setup\n\n## Installation\n\nInstall into your environment:\n\n```Shell\npip install django-controlled-vocabulary\n```\n\nAdd the app to the INSTALLED_APPS list in your Django settings file:\n\n```Python\nINSTALLED_APPS = [\n    ...\n    \'controlled_vocabulary\',\n    ...\n]\n```\n\nAdd the following path to your project urls.py:\n\n```Python\nfrom django.urls import path, include\n...\n\nurlpatterns = [\n    ...\n    path(\'vocabularies/\', include(\'controlled_vocabulary.urls\')),\n    ...\n]\n```\n\nRun the migrations:\n\n```Shell\n./manage.py migrate\n```\n\nDownload vocabulary data and add metadata to the database:\n\n```Shell\n./manage.py vocab init\n```\n\n## Configuration\n\n### Enabling specific vocabulary plug-ins (optional)\n\nCurrently all built-in plugins / managers are enabled by default. Add the following code in your settings.py to enable **only** specific vocabularies based on the import path of their classes. You can also use this to enable your own or third-party plugins.\n\n```Python\n# List of import paths to vocabularies lookup classes\nCONTROLLED_VOCABULARY_VOCABULARIES = [\n    \'controlled_vocabulary.vocabularies.iso639_2\',\n    \'controlled_vocabulary.vocabularies.dcmitype\',\n]\n```\n\nAfter enabling a new plug-in / manager, always run `./manage.py vocab init`.\n\n### ControlledTerm(s)Field\n\nUse the **ControlledTermField** field to define a field with an **autocomplete** to controlled terms in your Django Model:\n\n```Python\nfrom controlled_vocabulary.models import ControlledTermField\n\n...\nclass MyModel(models.Model):\n\n    ...\n    language_code = ControlledTermField(\n        \'iso639-2\',\n        null=True, blank=True\n    )\n```\n\nWhere `iso639-2` is the prefix of a controlled vocabulary in your database.\n\nControlledTermField is essentially syntactic sugar for a [ForeignKeyField](https://docs.djangoproject.com/en/3.0/ref/models/fields/#django.db.models.ForeignKey) with an adapted [Select](https://docs.djangoproject.com/en/3.0/ref/forms/widgets/#select) widget.\n\nFor multiple values, you can use **ControlledTermsField** (note the \'*s*\' in the name), which inherits from ManyToManyField with an adapted SelectMultiple widget. The useage is identical but obviously `null=True` should be omitted.\n\nBy default the widget proposes the given vocabulary to the end user, but they can use the dropdown to switch to any other available vocabulary (see screenshot at the top of this page). To **lock** the selection to a single vocabulary, use this expression instead:\n\n```Python\n    language_code = ControlledTermField(\n        [\'iso639-2\'],\n        null=True, blank=True\n    )\n```\n\nYou can have more than one prefix in that list if you want. The first item is always the one proposed by default on page load.\n\n# vocab (command line tool)\n\nvocab is a django command line tool that lets you manipulate the vocabularies\nand the plugins. To find out more use the help:\n\n```Shell\n./manage vocab help\n```\n\n',
    'author': 'geoffroy-noel-ddh',
    'author_email': 'geoffroy.noel@kcl.ac.uk',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kingsdigitallab/django-controlled-vocabulary',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
